@@ -1,6 +1,6 @@
 #include "libvoreutils.hpp"
 #include "util.hpp"
-#include <openssl/evp.h>
+#include <md5.h>
 #include <random>
 static const constexpr auto key_or_plain = [](auto && lhs, auto && rhs) {
 	static const constexpr auto key =
@@ -19,8 +19,8 @@ static const constexpr std::string_view DOWS_SET[] = {"Sun"sv, "Mon"sv, "Tue"sv,
 static const constexpr std::uint8_t MONTHS_SET[]   = {1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12};
 static const char * const MINUTES_RANGE            = "[0, 59]";
 static const char * const HOURS_RANGE              = "[0, 23]";
-static const char * const DAYS_RANGE               = "[1, 31]";
-static const char * const DOWS_RANGE               = "[mon, sun] or [1, 7]";
+static const char * const DAYS_RANGE               = "[1, 31] or 'L'";
+static const char * const DOWS_RANGE               = "[mon, sun] or [1, 7] or [1, 7]L";
 static const char * const MONTHS_RANGE             = "[jan, dec] or [1, 12]";
 
 #include "configuration.hpp"
@@ -151,6 +151,8 @@ struct Job {
 	std::set<std::uint8_t> timespec_month;    // 0-12                           or TIMESPEC_ASTERISK
 	std::optional<std::string_view> timespec_minute_raw, timespec_hour_raw, timespec_dom_raw, timespec_dow_raw, timespec_month_raw;  // for schedule for hashing
 	bool sunday_is_seven;
+	bool last_dom;
+	bool last_dow;
 	std::string schedule, schedule_raw;  // first for systemd, second for hashing
 	std::size_t boot_delay;
 	std::size_t start_hour;
@@ -227,6 +229,8 @@ struct Job {
 		this->valid           = true;
 		this->batch           = false;
 		this->sunday_is_seven = false;
+		this->last_dom        = false;
+		this->last_dow        = false;
 
 		this->cron_mail_success = cron_mail_success_t::dflt;
 		this->cron_mail_format  = cron_mail_format_t::dflt;
@@ -405,6 +409,14 @@ struct Job {
 		auto && dows          = this->parts[4];
 		this->timespec_minute = this->parse_time_unit<false, std::uint8_t>(minutes, "minute", MINUTES_SET, MINUTES_RANGE, int_map, this->timespec_minute_raw);
 		this->timespec_hour   = this->parse_time_unit<false, std::uint8_t>(hours, "hour", HOURS_SET, HOURS_RANGE, int_map, this->timespec_hour_raw);
+		this->last_dom        = days == "L"sv;
+		this->last_dow        = (dows.size() == 2 && dows.back() == 'L');
+		if(this->last_dom)
+			days = "1"sv;
+		else if(this->last_dow) {
+			days = "7/1"sv;
+			dows = dows.substr(0, 1);
+		}
 		this->timespec_dom    = this->parse_time_unit<false, std::uint8_t>(days, "day", DAYS_SET, DAYS_RANGE, int_map, this->timespec_dom_raw);
 		this->timespec_dow    = this->parse_time_unit<true, std::string_view>(dows, "dow", DOWS_SET, DOWS_RANGE, dow_map, this->timespec_dow_raw);
 		this->timespec_month  = this->parse_time_unit<false, std::uint8_t>(months, "month", MONTHS_SET, MONTHS_RANGE, month_map, this->timespec_month_raw);
@@ -675,8 +687,16 @@ struct Job {
 	if(this->persistent)    \
 	this->schedule_raw += what
 		timespec_comma(this->timespec_month, this->timespec_month_raw);
-		ADDBOTH('-');
+		if(this->last_dow || this->last_dom) {
+			ADDBOTH('~');
+		} else {
+			ADDBOTH('-');
+		}
 		timespec_comma(this->timespec_dom, this->timespec_dom_raw);
+		if (this->last_dow) {
+			ADDBOTH('/');
+			ADDBOTH('1');
+		}
 		ADDBOTH(' ');
 		timespec_comma(this->timespec_hour, this->timespec_hour_raw);
 		ADDBOTH(':');
@@ -824,34 +844,22 @@ struct Job {
 		assert(!this->jobid.empty());
 		this->unit_name = ("cron-"s += this->jobid) += '-';
 		if(!this->persistent) {
-		plain:
 			char buf[20 + 1];  // 18446744073709551615
 			this->unit_name += std::string_view{buf, static_cast<std::size_t>(std::snprintf(buf, sizeof(buf), "%" PRIu64 "", seq++))};
 		} else {
-#define TRY_CRYPTO(...) \
-	if(!__VA_ARGS__)      \
-	goto plain
-			static EVP_MD_CTX * evp = [] {
-				auto evp = EVP_MD_CTX_new();
-				assert(evp);
-				return evp;
-			}();
+			MD5_CTX ctx;
+			MD5Init(&ctx);
 
-			TRY_CRYPTO(EVP_DigestInit_ex(evp, EVP_md5(), nullptr));
-			TRY_CRYPTO(EVP_DigestUpdate(evp, this->schedule_raw.data(), this->schedule_raw.size()));
+			MD5Update(&ctx, reinterpret_cast<const std::uint8_t *>(this->schedule_raw.data()), this->schedule_raw.size());
 			for(auto && hunk : this->command) {
 				if(hunk.empty())
 					continue;
-				TRY_CRYPTO(EVP_DigestUpdate(evp, "", 1));  // NUL byte
-				TRY_CRYPTO(EVP_DigestUpdate(evp, hunk.data(), hunk.size()));
+				MD5Update(&ctx, reinterpret_cast<const std::uint8_t *>(""), 1);  // NUL byte
+				MD5Update(&ctx, reinterpret_cast<const std::uint8_t *>(hunk.data()), hunk.size());
 			}
 
-			std::uint8_t hash[128 / 8];
-			TRY_CRYPTO(EVP_DigestFinal(evp, hash, nullptr));
-
-			char buf[(sizeof(hash) * 2) + 1], *cur = buf;
-			for(auto b : hash)
-				cur += std::sprintf(cur, "%02" PRIx8 "", b);
+			char buf[((128 / 8) * 2) + 1];
+			MD5End(&ctx, buf);
 			this->unit_name += std::string_view{buf, sizeof(buf) - 1};
 		}
 	}
